import m from 'mithril';
import { Widget } from 'components/widgets/widget';
import { Pipe } from 'utilities/pipe';

require('./gauge.css');

export class Gauge extends Widget {
	private gaugeValue: Pipe<number>;
	private gaugeMaxValue: number;
	private unit: string;
	private decimals: number;
	
	protected bars: SVGGeometryElement[] = [];
	private integralValueElement: HTMLElement;
	private decimalValueElement: HTMLElement;
	
	private displayedValue = 0.0;
	private targetValue = 0.0;
	private valueChangeRate = 0.0;
	private litBars = 0;
	
	private animating = false;
	private shuttingDown = false;
	
	private lastGaugeUpdate = 0.0;
	private lastAnimationTick = 0.0;
	
	constructor(vnode: any) {
		super(vnode);
		
		this.gaugeValue = vnode.attrs.gaugeValue || new Pipe(0.0);
		this.gaugeMaxValue = vnode.attrs.gaugeMaxValue || 1.0;
		this.unit = vnode.attrs.unit || '';
		this.decimals = vnode.attrs.decimals || 0;
		
		this.gaugeValue.onChange(() => this.updateGauge());
	}
	
	onbeforeremove(vnode: m.Vnode<{}, {}>) {
		this.shuttingDown = true;
		super.onbeforeremove(vnode);
	}
	
	view(vnode: m.Vnode<{}, {}>): m.Children {
		return <div class="widget gauge" style={'flex: ' + this.widgetWidth}>
			<p><span class="integral-value"></span><span class="decimal-value"></span><span class="unit">{this.unit}</span></p>
			<svg>
			</svg>
		</div>;
	}
	
	updateGauge() {
		let now = Date.now() / 1000.0;
		
		let value = Math.max(0.0, Math.min(this.gaugeMaxValue, this.gaugeValue.get()));
		
		let valueStr = value.toFixed(this.decimals);
		let parts = valueStr.split('.');
		
		this.integralValueElement.innerText = parts[0];
		this.decimalValueElement.innerText = this.decimals > 0 ? '.' + parts[1] : '';
		
		let ratio = value / this.gaugeMaxValue;
		let numBars = this.bars.length;
		let threshold = 0.33 / numBars;
		
		if(ratio > this.targetValue + threshold || ratio < this.targetValue - threshold) {
			let animationTime = this.lastGaugeUpdate == 0.0 ? 0.0001 : Math.max(0.0001, Math.min(0.75, now - this.lastGaugeUpdate));
			
			this.valueChangeRate = (ratio - this.displayedValue) / animationTime;
			this.targetValue = ratio;
			
			this.enableAnimation();
		}
		
		this.lastGaugeUpdate = now;
	}
	
	private enableAnimation() {
		if(this.animating) return;
		this.animating = true;
		this.lastAnimationTick = Date.now() / 1000.0 - 0.016;
		requestAnimationFrame(() => this.animate());
	}
	
	private animate() {
		if(this.shuttingDown) return;
		
		if(this.valueChangeRate == 0.0) {
			this.animating = false;
			return;
		}
		
		requestAnimationFrame(() => this.animate());
		
		let now = Date.now() / 1000.0;
		if(now > this.lastAnimationTick + 0.75) {
			this.displayedValue = this.targetValue;
			this.valueChangeRate = 0.0;
		}
		else {
			let dt = now - this.lastAnimationTick;
			if(dt < 0.04) return; // limit framerate to save battery
			
			this.displayedValue += this.valueChangeRate * dt;
			if((this.displayedValue > this.targetValue) == (this.valueChangeRate > 0.0)) {
				this.displayedValue = this.targetValue;
				this.valueChangeRate = 0.0;
			}
		}
		
		this.lastAnimationTick = now;
		
		let numBars = this.bars.length;
		let litBars = Math.round(this.displayedValue * numBars);
		if(litBars == this.litBars)
			return;
		this.litBars = litBars;
		
		for(let barIdx = 0; barIdx < numBars; ++barIdx) {
			let bar = this.bars[barIdx];
			let isLit = barIdx < litBars;
			if(bar.classList.contains('lit') != isLit)
				bar.classList.toggle('lit', isLit);
		}
	}
	
	oncreate(vnode: any) {
		this.integralValueElement = vnode.dom.querySelector('span.integral-value');
		this.decimalValueElement = vnode.dom.querySelector('span.decimal-value');
		
		this.updateGauge();
	}
}
